/*-------------------------------------------------------------------------
 * Copyright (c) 2000-2007 Kenneth W. Sodemann (ken.sodemann@gmail.com)
 * Copyright (c) 2014-2019, Alex Panin (alex@linuxonly.ru) and Alexey Loginov (loginov.alex.valer@gmail.com)
 *-------------------------------------------------------------------------
 * mathwar_main
 *
 * Synopsis:
 *   Main module for the MathWar game.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *-------------------------------------------------------------------------
 */
#include <gtk/gtk.h>
#include <glib/gi18n.h>
#include <glib/gprintf.h>

#ifdef HAVE_CONFIG_H
     #include <config.h>
#endif

#include <stdlib.h>

#include "cards.h"
#include "ini_defs.h"
#include "mainwin.h"

#define STD_DISCLAIMER N_("\
%s version %s, Copyright (C) 2000-2007 Kenneth W. Sodemann\n\
Copyright (C) 2014-2019 Alex Panin and Alexey Loginov\n\n\
%s is free software; you can redistribute it and/or modify it\n\
under the terms of the GNU General Public License as published by\n\
the Free Software Foundation; either version 2 of the License, or\n\
(at your option) any later version.\n\n\
%s is distributed in the hope that it will be useful, but\n\
WITHOUT ANY WARRANTY; without even the implied warranty of\n\
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the\n\
GNU General Public License for more details.\n\n\
You should have received a copy of the GNU General Public License\n\
along with %s; if not, write to the Free Software Foundation, Inc.,\n\
51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA\n\n")

static void application_error_message(gchar *message)
{
	GtkWidget *dialog;
	
	dialog = gtk_message_dialog_new(NULL,
									GTK_DIALOG_MODAL,
									GTK_MESSAGE_ERROR,
									GTK_BUTTONS_CLOSE,
									message,
									NULL);
	
	gtk_dialog_run(GTK_DIALOG(dialog));
	gtk_widget_destroy(dialog);
}

static void application_startup(GApplication *application, gpointer data)
{
	/*Load pictures*/
	if (!load_deck()) {
		application_error_message("Error while loading game resources!");
		exit(EXIT_FAILURE);
	}
	/*Load settings*/
	ini_load_settings();
}

static void application_activate(GApplication *application, gpointer data)
{
	GList *appwindows;
	GtkWidget *mainwin;
	
	appwindows = gtk_application_get_windows(GTK_APPLICATION(application));
	
	if (appwindows == NULL) {
		/*Create application window and show it*/
		mainwin = create_mainwin(GTK_APPLICATION(application));
		gtk_application_add_window(GTK_APPLICATION(application), GTK_WINDOW(mainwin));
		gtk_widget_show(mainwin);
	} else {
		/*Windows already created, show main one*/
		gtk_window_present(GTK_WINDOW(appwindows->data));
	}
}

static void application_shutdown(GApplication *application, gpointer user_data)
{
	/*Save settings*/
	ini_close();
	/*Unload pictures*/
	destroy_deck();
}

int main (int argc, char *argv[])
{
	GtkApplication *app;
	gint res;
	gint show_copyright = FALSE;
	GOptionContext *context;
	GError *error = NULL;
	//GOptionGroup *group;
	GOptionEntry options [] =
	{ { "copyright", 'c', 0, G_OPTION_ARG_INT, &show_copyright, N_("Show copyright information"), N_("Show the copyright information and exit") },
      { NULL } };
   
#ifdef ENABLE_NLS
	bindtextdomain(PACKAGE, PACKAGE_LOCALE_DIR);
	textdomain(PACKAGE);
#endif
	context = g_option_context_new(_("- Simple \"flash-card\" type math game for kids"));
	g_option_context_add_main_entries(context, options, NULL);
	
	if (!g_option_context_parse(context, &argc, &argv, &error)) {
		g_printf(_("Unable to parse command line arguments: %s\n"), error->message);
		g_option_context_free(context);
		g_error_free(error);
		exit(EXIT_FAILURE);
	}
	
	g_option_context_free(context);
	
	if (show_copyright) {
		g_printf(STD_DISCLAIMER, PACKAGE, VERSION, PACKAGE, PACKAGE);
		exit(EXIT_SUCCESS);
	}
	
	app = gtk_application_new("org.gtk.MathWar", G_APPLICATION_FLAGS_NONE);
	
	g_signal_connect(app, "startup", G_CALLBACK(application_startup), NULL);
	g_signal_connect(app, "activate", G_CALLBACK(application_activate), NULL);
	g_signal_connect(app, "shutdown", G_CALLBACK(application_shutdown), NULL);
	
	res = g_application_run(G_APPLICATION(app), argc, argv);
	
	g_object_unref(app);
	
	return res;
}
