/*-------------------------------------------------------------------------
 * Copyright (c) 2000-2007 Kenneth W. Sodemann (ken.sodemann@gmail.com)
 * Copyright (c) 2014-2019, Alex Panin (alex@linuxonly.ru) and Alexey Loginov (loginov.alex.valer@gmail.com)
 *-------------------------------------------------------------------------
 * prop_dlg
 *
 * Synopsis:
 *   property dlg
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to
 * Free Software Foundation, Inc.
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 *
 *-------------------------------------------------------------------------
 */
#ifdef HAVE_CONFIG_H
   #include <config.h>
#endif

#include <gtk/gtk.h>
#include <glib/gi18n.h>

#include "draw.h"
#include "ini_defs.h"

/*
 * Definitions of important widget IDs
 */
#define NUM_ROUNDS_SPIN      "num_rounds"
#define SEC_PER_ROUND_SPIN   "secs_per_round"
#define SEC_BTWN_ROUND_SPIN  "secs_btwn_round"
#define RESP_DELAY_SPIN      "resp_delay"
#define ADD_PCT_SPIN         "add_pct"
#define SUB_PCT_SPIN         "sub_pct" 
#define MULT_PCT_SPIN        "mult_pct" 
#define MAX_ADD_SPIN         "max_add_spin"
#define MAX_SUB_SPIN         "max_sub_spin"
#define MAX_MULT_SPIN        "max_mult_spin"
#define GUESS_PCT_SPIN       "guess_pct_spin"
#define CORRECT_PCT_SPIN     "correct_pct_spin"
#define GUESS_DEV_SPIN       "guess_dev_spin"
#define GUESS_TIME_SPIN      "guess_time_spin"
#define GUESS_TIME_DEV_SPIN  "guess_time_dev_spin"
#define TITLE_FONT_PICKER    "title_font_picker"
#define TEXT_FONT_PICKER     "text_font_picker"
#define TEXT_COLOR_PICKER    "text_color_picker"

/*
 * Any widget that is associated with a configuration element will store
 * object data using these labels so we can determine what configuration
 * data needs to be changed.
 */
#define CONFIG_SECTION   "config_section"
#define CONFIG_DATA_EL   "config_data_elemnt"

/*
 * The percent spin buttons for the operators all need to add up to
 * 100%.  For that reason, when one is adjusted, the others will need
 * to be adjusted also.  Since there are three percent values in
 * the equation, each percent spin button will need a first and second
 * choice to adjust when changing the values.
 */
#define FIRST_ADJUST    "first_adjust_choice"
#define SECOND_ADJUST   "second_adjust_choice"

static void
on_response(GtkDialog *dialog, gint response_id, gpointer user_data)
{
	if (response_id == GTK_RESPONSE_APPLY) {
		ini_save_settings();
	} else {
		ini_forget_changes();
	}
	
	gtk_widget_hide(GTK_WIDGET(dialog));
}

/*
 * on_int_spin_button_changed
 *
 * This is the generic handler for value changes.  It simply saves the 
 * new value for the item changed.  Other items may require special
 * processing besides this.
 */
static void
on_int_spin_button_changed(GtkWidget *widget,
                            gpointer   user_data)
{
   gchar *section;
   gchar *label;
   gint   value;

   section = g_object_get_data(G_OBJECT(widget), CONFIG_SECTION);
   label = g_object_get_data(G_OBJECT(widget), CONFIG_DATA_EL);
   g_assert(section != NULL);
   g_assert(label != NULL);

   value = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));

   ini_set_int(section, label, value);
}


static void
on_pct_value_changed (GtkWidget *widget,
                      gpointer   user_data)
{
   GtkWidget   *first_pct_sb;
   GtkWidget   *second_pct_sb;
   gint         primary_pct;
   gint         first_adjust_pct;
   gint         second_adjust_pct;

   first_pct_sb = g_object_get_data(G_OBJECT(widget), FIRST_ADJUST);
   second_pct_sb = g_object_get_data(G_OBJECT(widget), SECOND_ADJUST);
   g_assert(first_pct_sb != NULL);
   g_assert(second_pct_sb != NULL);
   
   /*
    * Disable the change handlers for the scroll buttons we may end of
    * adjusting.
    */
   g_signal_handlers_block_by_func(G_OBJECT(first_pct_sb),
                                   G_CALLBACK(on_pct_value_changed),
                                   user_data);
   g_signal_handlers_block_by_func(G_OBJECT(second_pct_sb),
                                   G_CALLBACK(on_pct_value_changed),
                                   user_data);
   primary_pct = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
   first_adjust_pct = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(first_pct_sb));
   second_adjust_pct = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(second_pct_sb));

   /*
    * These items all need to add up to 100.  To get them back in line,
    * start with "first adjustment pct".  If it gets down to nothing, start
    * adjusting the "second adjustment pct".
    *
    * Note that since these values are stored in the config file, and
    * can be changed by the user, no assumptions should be made about
    * the three values adding up to 100 to begin with.
    */
   first_adjust_pct = 100 - (primary_pct + second_adjust_pct);
   if (first_adjust_pct < 0) {
	  first_adjust_pct = 0;
      second_adjust_pct = 100 - primary_pct;
      gtk_spin_button_set_value(GTK_SPIN_BUTTON (second_pct_sb), (gfloat)second_adjust_pct);      
      on_int_spin_button_changed(second_pct_sb, user_data);
   }
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(first_pct_sb), (gfloat)first_adjust_pct);      
   on_int_spin_button_changed (first_pct_sb, user_data);
   g_assert((primary_pct + first_adjust_pct + second_adjust_pct) == 100);
   
   g_signal_handlers_unblock_by_func(G_OBJECT(first_pct_sb),
	                                 G_CALLBACK(on_pct_value_changed),
	                                 user_data);
   g_signal_handlers_unblock_by_func(G_OBJECT(second_pct_sb),
                                     G_CALLBACK(on_pct_value_changed),
                                     user_data);

   on_int_spin_button_changed (widget, user_data);
}


static void
on_font_set(GtkFontButton *widget, gpointer user_data)
{
   gchar *section;
   gchar *label;
   const gchar *fontstr;

   section = g_object_get_data(G_OBJECT(widget), CONFIG_SECTION);
   label = g_object_get_data(G_OBJECT(widget), CONFIG_DATA_EL);
   g_assert(section != NULL);
   g_assert(label != NULL);
   
   fontstr = gtk_font_button_get_font_name(widget);
   if (fontstr != NULL) {
	   ini_set_string(section, label, fontstr);
   }
}


static void
on_text_color_set(GtkColorButton *widget, gpointer user_data)
{
	GdkRGBA rgba;
	gchar *colorstr;
	
	gtk_color_chooser_get_rgba(GTK_COLOR_CHOOSER(widget), &rgba);
	
	colorstr = gdk_rgba_to_string((const GdkRGBA *)&rgba);
	
	if (colorstr != NULL) {
		ini_set_string(FONTS, FONT_COLOR, colorstr);
		g_free(colorstr);
	}
}

static void 
create_general_tab (GtkNotebook *notebook)
{
   GtkWidget   *tab_lbl;
   GtkWidget   *grid;
   GtkWidget   *label;
   GtkAdjustment   *num_rounds_adj;
   GtkWidget   *num_rounds;
   GtkAdjustment   *sec_per_round_adj;
   GtkWidget   *sec_per_round;
   GtkAdjustment   *sec_btwn_round_adj;
   GtkWidget   *sec_btwn_round;
   GtkAdjustment   *resp_delay_adj;
   GtkWidget   *resp_delay;
   GtkWidget   *title_fp;
   GtkWidget   *text_fp;
   GtkWidget   *text_cp;
   gchar       *strval;
   gint         value;
   GdkRGBA rgba;
   
   /*
    * Labels
    */
   tab_lbl = gtk_label_new(_("General"));
   
   grid = gtk_grid_new();
   gtk_container_set_border_width(GTK_CONTAINER(grid), 5);
   gtk_grid_set_row_spacing(GTK_GRID(grid), 3);
   gtk_grid_set_column_spacing(GTK_GRID(grid), 5);

   label = gtk_label_new (_("Number of Rounds:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1);
   
   label = gtk_label_new (_("Seconds per Round:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 0, 1, 1, 1);

   label = gtk_label_new (_("Title Text Font:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 0, 2, 1, 1);

   label = gtk_label_new (_("Text Font:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 0, 3, 1, 1);
   
   label = gtk_label_new (_("Text Color:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 0, 4, 1, 1);

   label = gtk_label_new (_("Seconds Between Rounds:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 2, 0, 1, 1);

   label = gtk_label_new (_("Response Delay:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 2, 1, 1, 1);
   
   /*
    * Spin buttons and associated adjustments.
    */
   value = ini_get_int(BASIC_PARMS, NUM_ROUNDS, DEF_NUM_ROUNDS);
   num_rounds_adj = gtk_adjustment_new(value, MIN_ROUNDS, MAX_ROUNDS, 1, 10, 10);
   num_rounds = gtk_spin_button_new(GTK_ADJUSTMENT(num_rounds_adj), 1, 0);
   g_object_set_data(G_OBJECT(num_rounds), CONFIG_SECTION, BASIC_PARMS);
   g_object_set_data(G_OBJECT(num_rounds), CONFIG_DATA_EL, NUM_ROUNDS);
   g_signal_connect(G_OBJECT(num_rounds), "changed", G_CALLBACK(on_int_spin_button_changed), NULL);
   gtk_grid_attach(GTK_GRID(grid), num_rounds, 1, 0, 1, 1);
   gtk_widget_set_tooltip_text(num_rounds, _("The number of rounds in a game.  Changes will not take effect until the next new game."));
   
   value = ini_get_int(BASIC_PARMS, SECONDS, DEF_SECONDS);
   sec_per_round_adj = gtk_adjustment_new(value, MIN_SECS, MAX_SECS, 1, 10, 10);
   sec_per_round = gtk_spin_button_new(GTK_ADJUSTMENT(sec_per_round_adj), 1, 0);
   g_object_set_data(G_OBJECT(sec_per_round), CONFIG_SECTION, BASIC_PARMS);
   g_object_set_data(G_OBJECT(sec_per_round), CONFIG_DATA_EL, SECONDS);
   g_signal_connect(G_OBJECT(sec_per_round), "changed", G_CALLBACK(on_int_spin_button_changed), NULL);
   gtk_grid_attach(GTK_GRID(grid), sec_per_round, 1, 1, 1, 1);
   gtk_widget_set_tooltip_text(sec_per_round, _("The number of seconds in a round.  Changes will not take effect until the next new game."));

   value = ini_get_int(BASIC_PARMS, SECONDS_BTWN, DEF_SECS_BTWN);
   sec_btwn_round_adj = gtk_adjustment_new(value, MIN_DEL_SECS, MAX_DEL_SECS, 1, 10, 10);
   sec_btwn_round = gtk_spin_button_new(GTK_ADJUSTMENT(sec_btwn_round_adj), 1, 0);
   g_object_set_data(G_OBJECT(sec_btwn_round), CONFIG_SECTION, BASIC_PARMS);
   g_object_set_data(G_OBJECT(sec_btwn_round), CONFIG_DATA_EL, SECONDS_BTWN);
   g_signal_connect(G_OBJECT(sec_btwn_round), "changed", G_CALLBACK(on_int_spin_button_changed), NULL);
   gtk_grid_attach(GTK_GRID(grid), sec_btwn_round, 3, 0, 1, 1);
   gtk_widget_set_tooltip_text(sec_btwn_round, _("The number of seconds that the cards are shown face down before each round."));
   
   value = ini_get_int(BASIC_PARMS, RESP_DELAY, DEF_RESP_DELAY);
   resp_delay_adj = gtk_adjustment_new(value, MIN_DEL_SECS, MAX_DEL_SECS, 1, 10, 10);
   resp_delay = gtk_spin_button_new(GTK_ADJUSTMENT(resp_delay_adj), 1, 0);
   g_object_set_data(G_OBJECT(resp_delay), CONFIG_SECTION, BASIC_PARMS);
   g_object_set_data(G_OBJECT(resp_delay), CONFIG_DATA_EL, RESP_DELAY);
   g_signal_connect(G_OBJECT(resp_delay), "changed", G_CALLBACK(on_int_spin_button_changed), NULL);
   gtk_grid_attach(GTK_GRID(grid), resp_delay, 3, 1, 1, 1);
   gtk_widget_set_tooltip_text(resp_delay, _("The number of seconds that most messages are displayed.  This only applies to messages that do not require any input from the player."));
   
   /*
    * Font pickers
    */
   title_fp = gtk_font_button_new();
   gtk_font_button_set_show_style(GTK_FONT_BUTTON(title_fp), TRUE);
   gtk_font_button_set_show_size(GTK_FONT_BUTTON(title_fp), TRUE);
   strval = ini_get_string(FONTS, TITLE_FONT, DEF_TITLE_FONT);
   gtk_font_button_set_font_name(GTK_FONT_BUTTON(title_fp), (const gchar *)strval);
   g_free (strval);
   g_object_set_data(G_OBJECT(title_fp), CONFIG_SECTION, FONTS);
   g_object_set_data(G_OBJECT(title_fp), CONFIG_DATA_EL, TITLE_FONT);
   g_signal_connect(G_OBJECT(title_fp), "font-set", G_CALLBACK(on_font_set), NULL);
   gtk_grid_attach(GTK_GRID(grid), title_fp, 1, 2, 3, 1);
   gtk_widget_set_tooltip_text(title_fp, _("Choose the font used to display title text.  Changes take effect the next time text is drawn."));
   
   text_fp = gtk_font_button_new();
   gtk_font_button_set_show_style(GTK_FONT_BUTTON(text_fp), TRUE);
   gtk_font_button_set_show_size(GTK_FONT_BUTTON(text_fp), TRUE);
   strval = ini_get_string(FONTS, TEXT_FONT, DEF_TEXT_FONT);
   gtk_font_button_set_font_name(GTK_FONT_BUTTON(text_fp), (const gchar *)strval);
   g_free(strval);
   g_object_set_data(G_OBJECT(text_fp), CONFIG_SECTION, FONTS);
   g_object_set_data(G_OBJECT(text_fp), CONFIG_DATA_EL, TEXT_FONT);
   g_signal_connect(G_OBJECT(text_fp), "font-set", G_CALLBACK(on_font_set), NULL);
   gtk_grid_attach(GTK_GRID(grid), text_fp, 1, 3, 3, 1);
   gtk_widget_set_tooltip_text(text_fp, _("Choose the font used to display normal text.  Changes take effect the next time text is drawn."));
   
   /*
    * Color Picker
    */
   text_cp = gtk_color_button_new();
   strval = ini_get_string(FONTS, FONT_COLOR, DEF_COLOR);
   if (gdk_rgba_parse(&rgba, (const gchar *)strval)) {
	   gtk_color_chooser_set_rgba(GTK_COLOR_CHOOSER(text_cp), (const GdkRGBA *)&rgba);
   }
   g_free(strval);
   g_signal_connect(G_OBJECT(text_cp), "color-set", G_CALLBACK(on_text_color_set), NULL);
   gtk_grid_attach(GTK_GRID(grid), text_cp, 1, 4, 3, 1);
   gtk_widget_set_tooltip_text(text_cp, _("Choose the color used for drawing text.  Changes take effect the next time text is drawn."));
   
   gtk_notebook_append_page(notebook, grid, tab_lbl);
}


static void 
create_comp_player_tab (GtkNotebook *notebook)
{
   GtkWidget   *tab_lbl;
   GtkWidget   *grid;
   GtkWidget   *label;
   GtkAdjustment   *adjustment;
   GtkWidget   *spinner;
   gint         value;
   
   tab_lbl = gtk_label_new(_("Computer Player"));
   
   grid = gtk_grid_new();
   gtk_container_set_border_width(GTK_CONTAINER(grid), 5);
   gtk_grid_set_row_spacing(GTK_GRID(grid), 3);
   gtk_grid_set_column_spacing(GTK_GRID(grid), 5);
   
   label = gtk_label_new(_("Computer Guess Percentage:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1);
   
   label = gtk_label_new(_("Percent Guessed Correctly:"));
   gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 0, 1, 1, 1);
   
   label = gtk_label_new (_("Maximum Guess Deviation:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 0, 2, 1, 1);
   
   label = gtk_label_new(_("Guess Time:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 2, 0, 1, 1);
   
   label = gtk_label_new (_("Guess Time Deviation:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 2, 1, 1, 1);
   
   value = ini_get_int(COMP_PLAYER, GUESS_PCT, DEF_GUESS_PCT);
   adjustment = gtk_adjustment_new(value, MIN_PCT, MAX_PCT, 1, 10, 10);
   spinner = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1, 0);
   g_object_set_data(G_OBJECT(spinner), CONFIG_SECTION, COMP_PLAYER);
   g_object_set_data(G_OBJECT(spinner), CONFIG_DATA_EL, GUESS_PCT);
   g_signal_connect(G_OBJECT(spinner), "changed", G_CALLBACK(on_int_spin_button_changed), NULL);
   gtk_grid_attach(GTK_GRID(grid), spinner, 1, 0, 1, 1);
   gtk_widget_set_tooltip_text(spinner, _("The average percent of rounds that the computer will make a guess (assuming the human player does not guess first).  Changes take effect in the next new game."));
   
   value = ini_get_int(COMP_PLAYER, RIGHT_PCT, DEF_RIGHT_PCT);
   adjustment = gtk_adjustment_new(value, MIN_PCT, MAX_PCT, 1, 10, 10);
   spinner = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 1, 0);
   g_object_set_data(G_OBJECT(spinner), CONFIG_SECTION, COMP_PLAYER);
   g_object_set_data(G_OBJECT(spinner), CONFIG_DATA_EL, RIGHT_PCT);
   g_signal_connect(G_OBJECT(spinner), "changed", G_CALLBACK(on_int_spin_button_changed), NULL);
   gtk_grid_attach(GTK_GRID(grid), spinner, 1, 1, 1, 1);
   gtk_widget_set_tooltip_text(spinner, _("The average percent of guesses that the computer will be correct.  Changes take effect in the next new game."));
   
   value = ini_get_int(COMP_PLAYER, GUESS_DEV, DEF_GUESS_DEV);
   adjustment = gtk_adjustment_new(value, MIN_DEV_VALUE, MAX_DEV_VALUE, 1, 10, 10);
   spinner = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 1, 0);
   g_object_set_data(G_OBJECT(spinner), CONFIG_SECTION, COMP_PLAYER);
   g_object_set_data(G_OBJECT(spinner), CONFIG_DATA_EL, GUESS_DEV);
   g_signal_connect(G_OBJECT(spinner), "changed", G_CALLBACK(on_int_spin_button_changed), NULL);
   gtk_grid_attach(GTK_GRID(grid), spinner, 1, 2, 1, 1);
   gtk_widget_set_tooltip_text(spinner, _("Creates a window that the computer might be incorrect by when making an incorrect guess.  Changes take effect in the next new game."));
   
   value = ini_get_int(COMP_PLAYER, GUESS_TIME, DEF_TIME);
   adjustment = gtk_adjustment_new(value, MIN_PCT, MAX_PCT, 1, 10, 10);
   spinner = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 1, 0);
   g_object_set_data(G_OBJECT(spinner), CONFIG_SECTION, COMP_PLAYER);
   g_object_set_data(G_OBJECT(spinner), CONFIG_DATA_EL, GUESS_TIME);
   g_signal_connect(G_OBJECT(spinner), "changed", G_CALLBACK(on_int_spin_button_changed), NULL);
   gtk_grid_attach(GTK_GRID(grid), spinner, 3, 0, 1, 1);
   gtk_widget_set_tooltip_text(spinner, _("Average time that the computer player will wait before making a guess, expressed as a percent of the round time.  Changes take effect in the next new game."));
   
   value = ini_get_int(COMP_PLAYER, MAX_DEVIATION, DEF_MAX_DEV);
   adjustment = gtk_adjustment_new(value, MIN_DEV_VALUE, MAX_DEV_VALUE, 1, 10, 10);
   spinner = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 1, 0);
   g_object_set_data(G_OBJECT(spinner), CONFIG_SECTION, COMP_PLAYER);
   g_object_set_data(G_OBJECT(spinner), CONFIG_DATA_EL, MAX_DEVIATION);
   g_signal_connect(G_OBJECT(spinner), "changed", G_CALLBACK(on_int_spin_button_changed), NULL);
   gtk_grid_attach(GTK_GRID(grid), spinner, 3, 1, 1, 1);
   gtk_widget_set_tooltip_text(spinner, _("Creates a window within which the computer player will guess.  Changes take effect in the next new game."));
   
   gtk_notebook_append_page(notebook, grid, tab_lbl);
}


static void 
create_operator_tab(GtkNotebook *notebook)
{
   GtkWidget   *tab_lbl;
   GtkWidget   *grid;
   GtkWidget   *label;
   GtkAdjustment   *adjustment;
   GtkWidget   *spinner;
   GtkWidget   *add_spinner;
   GtkWidget   *sub_spinner;
   GtkWidget   *mult_spinner;
   gint         value;
   
   tab_lbl = gtk_label_new (_("Operators"));
   
   grid = gtk_grid_new();
   gtk_container_set_border_width(GTK_CONTAINER(grid), 5);
   gtk_grid_set_row_spacing(GTK_GRID(grid), 3);
   gtk_grid_set_column_spacing(GTK_GRID(grid), 5);

   label = gtk_label_new(_("Addition Percent:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 0, 0, 1, 1);
   
   label = gtk_label_new(_("Subtraction Percent:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 0, 1, 1, 1);
   
   label = gtk_label_new(_("Multiplication Percent:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 0, 2, 1, 1);
   
   label = gtk_label_new(_("Max Addition Number:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 2, 0, 1, 1);
   
   label = gtk_label_new(_("Max Subtraction Number:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 2, 1, 1, 1);
   
   label = gtk_label_new(_("Max Multiplication Number:"));
   gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_RIGHT);
   gtk_widget_set_halign(label, GTK_ALIGN_END);
   gtk_widget_set_valign(label, GTK_ALIGN_CENTER);
   gtk_grid_attach(GTK_GRID(grid), label, 2, 2, 1, 1);
   
   value = ini_get_int(OPERATORS, ADD_PCT, DEF_ADD_PCT);
   adjustment = gtk_adjustment_new(value, MIN_PCT, MAX_PCT, 1, 10, 10);
   add_spinner = gtk_spin_button_new(GTK_ADJUSTMENT (adjustment), 1, 0);
   g_object_set_data(G_OBJECT(add_spinner), CONFIG_SECTION, OPERATORS);
   g_object_set_data(G_OBJECT(add_spinner), CONFIG_DATA_EL, ADD_PCT);
   g_signal_connect(G_OBJECT(add_spinner), "changed", G_CALLBACK(on_pct_value_changed), NULL);
   gtk_grid_attach(GTK_GRID(grid), add_spinner, 1, 0, 1, 1);
   gtk_widget_set_tooltip_text(add_spinner, _("The percent of rounds that will be addition problems.  Changes take effect in the next new game."));
   
   value = ini_get_int(OPERATORS, SUB_PCT, DEF_SUB_PCT);
   adjustment = gtk_adjustment_new(value, MIN_PCT, MAX_PCT, 1, 10, 10);
   sub_spinner = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 1, 0);
   g_object_set_data(G_OBJECT(sub_spinner), CONFIG_SECTION, OPERATORS);
   g_object_set_data(G_OBJECT(sub_spinner), CONFIG_DATA_EL, SUB_PCT);
   g_signal_connect(G_OBJECT(sub_spinner), "changed", G_CALLBACK(on_pct_value_changed), NULL);
   gtk_grid_attach(GTK_GRID(grid), sub_spinner, 1, 1, 1, 1);
   gtk_widget_set_tooltip_text(sub_spinner, _("The percent of rounds that will be subtraction problems.  Changes take effect in the next new game."));
   
   value = ini_get_int(OPERATORS, MULT_PCT, DEF_MULT_PCT);
   adjustment = gtk_adjustment_new(value, MIN_PCT, MAX_PCT, 1, 10, 10);
   mult_spinner = gtk_spin_button_new(GTK_ADJUSTMENT (adjustment), 1, 0);
   g_object_set_data(G_OBJECT(mult_spinner), CONFIG_SECTION, OPERATORS);
   g_object_set_data(G_OBJECT(mult_spinner), CONFIG_DATA_EL, MULT_PCT);
   g_signal_connect(G_OBJECT(mult_spinner), "changed", G_CALLBACK(on_pct_value_changed), NULL);
   gtk_grid_attach(GTK_GRID(grid), mult_spinner, 1, 2, 1, 1);
   gtk_widget_set_tooltip_text(mult_spinner, _("The percent of rounds that will be multiplication problems.  Changes take effect in the next new game."));
   
   /*
    * Now that all the pct spinners have been created, we need to link 
    * them so that each one knows its first and second choices to 
    * adjust when their own values change.
    *
    * The scheme should be cyclic, top to bottom, as on the screen.
    */
   g_object_set_data(G_OBJECT(add_spinner), FIRST_ADJUST, sub_spinner);
   g_object_set_data(G_OBJECT(add_spinner), SECOND_ADJUST, mult_spinner);
   
   g_object_set_data(G_OBJECT(sub_spinner), FIRST_ADJUST, mult_spinner);
   g_object_set_data(G_OBJECT(sub_spinner), SECOND_ADJUST, add_spinner);
   
   g_object_set_data(G_OBJECT(mult_spinner), FIRST_ADJUST, add_spinner);
   g_object_set_data(G_OBJECT (mult_spinner), SECOND_ADJUST, sub_spinner);

   value = ini_get_int(OPERATORS, ADD_MAX, DEF_ADD_MAX);
   adjustment = gtk_adjustment_new (value, MIN_NUMBER, MAX_NUMBER, 1, 10, 10);
   spinner = gtk_spin_button_new (GTK_ADJUSTMENT (adjustment), 1, 0);
   g_object_set_data(G_OBJECT(spinner), CONFIG_SECTION, OPERATORS);
   g_object_set_data(G_OBJECT(spinner), CONFIG_DATA_EL, ADD_MAX);
   g_signal_connect(G_OBJECT(spinner), "changed", G_CALLBACK(on_int_spin_button_changed), NULL);
   gtk_grid_attach(GTK_GRID(grid), spinner, 3, 0, 1, 1);
   gtk_widget_set_tooltip_text(spinner, _("The largest card value that will be used for addition problems.  Changes take effect in the next new game."));
   
   value = ini_get_int(OPERATORS, SUB_MAX, DEF_SUB_MAX);
   adjustment = gtk_adjustment_new(value, MIN_NUMBER, MAX_NUMBER, 1, 10, 10);
   spinner = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment), 1, 0);
   g_object_set_data(G_OBJECT(spinner), CONFIG_SECTION, OPERATORS);
   g_object_set_data(G_OBJECT(spinner), CONFIG_DATA_EL, SUB_MAX);
   g_signal_connect(G_OBJECT(spinner), "changed", G_CALLBACK(on_int_spin_button_changed), NULL);
   gtk_grid_attach(GTK_GRID(grid), spinner, 3, 1, 1, 1);
   gtk_widget_set_tooltip_text(spinner, _("The largest card value that will be used for subtraction problems.  Changes take effect in the next new game."));
   
   value = ini_get_int(OPERATORS, MULT_MAX, DEF_MULT_MAX);
   adjustment = gtk_adjustment_new(value, MIN_NUMBER, MAX_NUMBER, 1, 10, 10);
   spinner = gtk_spin_button_new(GTK_ADJUSTMENT (adjustment), 1, 0);
   g_object_set_data(G_OBJECT(spinner), CONFIG_SECTION, OPERATORS);
   g_object_set_data(G_OBJECT(spinner), CONFIG_DATA_EL, MULT_MAX);
   g_signal_connect(G_OBJECT(spinner), "changed", G_CALLBACK(on_int_spin_button_changed), NULL);
   gtk_grid_attach(GTK_GRID(grid), spinner, 3, 2, 1, 1);
   gtk_widget_set_tooltip_text(spinner, _("The largest card value that will be used for multiplication problems.  Changes take effect in the next new game."));
   
   gtk_notebook_append_page(notebook, grid, tab_lbl);
}


static GtkWidget *
create_prop_dlg (GtkWidget *parent)
{
   GtkWidget *notebook, *dialog, *content_area;
   
   notebook = gtk_notebook_new();
   create_general_tab(GTK_NOTEBOOK(notebook));
   create_comp_player_tab(GTK_NOTEBOOK(notebook));
   create_operator_tab(GTK_NOTEBOOK(notebook));

   dialog = gtk_dialog_new_with_buttons(_("Preferences"),
                                         GTK_WINDOW(parent),
                                         GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
                                         _("_Apply"),
                                         GTK_RESPONSE_APPLY,
                                         _("_Close"),
                                         GTK_RESPONSE_CLOSE,
                                         NULL);
   
   content_area = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
   
   g_signal_connect(dialog, "response", G_CALLBACK(on_response), dialog);
   g_signal_connect(dialog, "delete-event", G_CALLBACK(gtk_widget_hide_on_delete), NULL);

   gtk_container_add(GTK_CONTAINER(content_area), notebook);
   gtk_widget_show_all(dialog);
   
   return dialog;
}


void
show_prop_dlg (GtkWidget *parent)
{
   static GtkWidget *prop_dlg = NULL;

   /*
    * If the dialog box exists, just make sure it is visable, otherwise
    * create it.
    */
   if (prop_dlg != NULL) {
	   gtk_window_present(GTK_WINDOW(prop_dlg));
	} else {
      prop_dlg = create_prop_dlg(parent);
   }
}
